lorom

;************************
; Tiles builder PLM
; Version 1.0

; By: Benox50

; High = PLM instructions (level data edits)
; Low = Event for those edits to happen, zero for always

; Use this to edit the level data instead of using room states and save space. This both check an event, then an index to PLM instructions list to be used to draw tiles. It comes with some basic presets, but you must know some PLM asm to add new ones.

; With some smart moves you can reuse draw instructions and make your world more dynamic without wasting ton of level data for small changes.
;************************

;This will also help you understand how PLMs work, but basic asm knowledge is needed
;Bonus, this comes packed with some new generic PLM instructions


;.........................................



;PLM header
;Room tiles or builder PLM
org $84F680 ; <- F00F is the PLM ID in Smile or your SM editor
	DW #PLM_Build_Init, #PLM_Build_Inst










; FREE SPACE $84 ; *Repoint this if you need
;org $84F340



;;;;;;;;;;;;;;;;;;;;;; This utility PLM edits level data when a said event occurs
;;; Room tiles PLM ;;; Alternative to room state to save space, act as a bunch of hard coded PLMs entries
;;;;;;;;;;;;;;;;;;;;;; Benox50
{
;;PLM settings, XX YY ;;

; XX Index to PLM instructions and their draw instructions
; YY Event state that triggers the tiles change, 0 for none

;Default nothing PLM instructions list
PLM_Build_Inst:
	DW $86B4 ;Sleep (wait)

;Initiation
PLM_Build_Init:
print " PLM_Build (Initiation): ", pc
	PHX
;Use PLM setting low as event to check
	LDA $1DC7,y : AND #$00FF : BEQ +
	JSL $808233 : BCC ++ ;Cool vanilla routine to check an event
+
;Use PLM setting high as index to pick corresponding instruction list
	LDA $1DC8,y : AND #$00FF : ASL : TAX
	LDA.w PLM_Build_InitArr,x : STA $1D27,y
++
	PLX : RTS
;Instructions list array (from PLM setting this represent as 00,01,02 for list A,B,C)
PLM_Build_InitArr:
	DW #PLM_Build_InstA, #PLM_Build_InstB, #PLM_Build_InstC  ;You can put more instruction list here




;;; Instructions and their draws to edit level data ;;; (those tell what to do, you can call draw instructions to draw tiles, move the PLM around to draw elswhere n do bunch of things, those examples will help understand)

;Clear 4 rows of 6 blocks and draw blue door cap on right side (condition to not draw door if samus was near PLM, this was to prevent crash when samus exits said door into the PLM, also a good example of what you can do)
PLM_Build_InstA:
	DW $0001, #PLM_Build_Draw1 ;Draw empty air rows
	DW #PLM_MoveRM : DB $06 ;Move PLM 6 blocks right
	DW $8D41 : DB $04,$02 : DW #PLM_Build_InstA2 ;Compare Samus pos to PLM, Goto "PLM_Build_InstA2" if samus near PLM
	DW $0001, $A9B3 ;Draw blue door cap
	DW $8AF1 : DB $40 ;Set PLM block BTS
PLM_Build_InstA2:
	DW $86BC ;Delete PLM (we finished)

;Clear 4 rows of 6 blocks and draw blue door cap on left side
PLM_Build_InstB:
	DW $0001, #PLM_Build_Draw1 ;Draw empty air rows
	DW #PLM_MoveL ;Move PLM left
	DW $8D41 : DB $04,$02 : DW #PLM_Build_InstB2 ;Samus pos to PLM, Prevent crash (door exit stuff)
	DW $0001, $A9EF ;Draw blue door cap
	DW $8AF1 : DB $41 ;Set PLM block BTS
PLM_Build_InstB2:
	DW $86BC ;Delete PLM

;Clear 3 rows of 6 blocks 
PLM_Build_InstC:
	DW $0001, #PLM_Build_Draw2 ;Draw empty air row
	DW #PLM_MoveD ;Move PLM down
	DW $0001, #PLM_Build_Draw2 ;Draw empty air row
	DW #PLM_MoveD ;Move PLM down
	DW $0001, #PLM_Build_Draw2 ;Draw empty air row
	DW $86BC ;Delete PLM



;;; Draw Instructions ;;; (Those actually draw tiles in realtime, this is the format to use)

;Clear 4 rows of 6 blocks
PLM_Build_Draw1:
	DW $0006,$00FF,$00FF,$00FF,$00FF,$00FF,$00FF ;First value for number of tiles (0000 draw right, 8000 draw down), then next values (XYYY) represent X as the block type and Y as tile to use from TileTable
	DB $00,$01 ;This draw next entry with an offset from PLM position (PosX, PosY)
	DW $0006,$00FF,$00FF,$00FF,$00FF,$00FF,$00FF
	DB $00,$02
	DW $0006,$00FF,$00FF,$00FF,$00FF,$00FF,$00FF
	DB $00,$03
	DW $0006,$00FF,$00FF,$00FF,$00FF,$00FF,$00FF
	DW $0000 ;Terminator (we finished)

;Clear 1 row of 6 blocks
PLM_Build_Draw2:
	DW $0006,$00FF,$00FF,$00FF,$00FF,$00FF,$00FF
	DW $0000
	
}







;;;;;;;;;;;;;;;;;
;; PLM Generic ;;
;;;;;;;;;;;;;;;;;
{

;;; Generic inst. for moving PLM 1 block (*Can also be used as standard code, like can JSR to it in any PLM run code);;;

;Move PLM Up by 1;
org $848D89
PLM_MoveU:
	LDA $1C87,x
	SEC : SBC $07A5 : SBC $07A5
	STA $1C87,x
	RTS

;Move PLM Down by 1;
org $84AB00
PLM_MoveD:
	LDA $07A5
	ASL : CLC : ADC $1C87,x
	STA $1C87,x
	RTS
	
;Move PLM right by 1;
org $84ABD6
PLM_MoveR:
	INC $1C87,x : INC $1C87,x
	RTS

;Move PLM left by 1;
org $848D98
PLM_MoveL:
	DEC $1C87,x : DEC $1C87,x
	RTS

;;; Generic inst. for moving PLM multiple blocks ;;;
{
; PLM_MoveRM:
	; LDA $0000,y : AND #$00FF : ASL
	; CLC : ADC $1C87,x : STA $1C87,x
	; INY : RTS
; PLM_MoveLM:
	; LDA $0000,y : AND #$00FF : ASL : STA $16
	; LDA $1C87,x : SEC : SBC $16 : STA $1C87,x
	; INY : RTS
; PLM_MoveDM:
	; PHY
	; LDA $0000,y : AND #$00FF : TAY
; -	
	; LDA $07A5
	; ASL : CLC : ADC $1C87,x
	; STA $1C87,x
	; DEY : BNE -
	; PLY
	; INY : RTS
; PLM_MoveUM:
	; PHY
	; LDA $0000,y : AND #$00FF : TAY
; -	
	; LDA $1C87,x
	; SEC : SBC $07A5 : SBC $07A5
	; STA $1C87,x
	; DEY : BNE -
	; PLY
	; INY : RTS

}
