;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;																				|
; Spotlight effect on Samus ASM  [Filename: Spotlight v.1.00.asm] (version 1.00 ; september 2020)								|
; =============================																	|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;																				|
; Description:																			|
; - This patch create a new FX1 type which darkens the room where it is apllied.										|
; - In such a room, Samus appear surounded by a glowing luminous halo.												|
; - When she moves, the halo is shifted in the direction of displacement.											|
;																				|
; Warning:																			|
; - The patch replace the original FX1 Fog effect. Don't apply this patch if you plan to use fog in your hack !							|
;																				|
; How to install ?:																		|
; - Read info, control free space usage and apply the patch "Subroutine_Array_v.1.00.asm" prior to this patch, but ...						|
;   "Subroutine array" is a prerequisite for several patches by BigDomino. It should only be applied ONCE to your hack and never again ! 			|
;   If you alrerady have applied "Subroutine array" for another BigDomino patch, it's done ! Go to the next step.						|
;   Don't apply it again since it would reset some pointers, making older patches ineffectives.									|
; - Control free space usage for this patch (see below).													|
; - Apply this ASM patch to your unheadered ROM.														|
; - Done.																			|
;																				|
; How to use in a room ? (Smile 2.5):																|
; - In a room with FX1 valid pointer, open the FX1 editor and choose: 0C - Fog.											|
; - Set palette blend value to 06, 0C, 12 or 18 (-> the higher, the darker).											|
; - Uncheck all options in "Palette FX" tab.															|
; - In the "A/B" tab, set the "A" parameter to 28 & "B" parameter to 30.											|
; - Done.																			|
;																				|
; Limitation, regarding enemy sprites:																|
; - When enemies are used in a room with the halo effect, they should use palette line 0007 (ie: in the bottom right corner of the "Species editor", 		|
;   see the column next to enemy ID and double-clic to edit).													|
;   Due to SNES limitation, only the enemies whose color palette is assigned to palette line index 7 of CGRAM are correctly affected by the halo effect ;	|
;   Those whose palette is assigned to another palette line (ie: 0001 ; 0002 or 0003) are left unaffected (ie: They are not darkened at all).			|
;   In that case, the enemies' sprites will appears much brighter than they should, in total contrast with the dark room all around...				|
;																				|
;   So, for rooms where only one type of enemy is needed, there is no problem : use palette line 0007.								|
;   In the other hand, if more than one type of enemy is needed, a possible workaround is to assign all enemies to palette line 0007 in the species editor.	|
;   In that case, all enemies will share the specific color palette of the enemy located at the very bottom of the list (nice & unusual effect are possible).	|
;   NB: In a previously designed room, since the order in this list is decisive, don't hesitate to remove all enemies ID by double-clicking them,		|
;   then, re-add each species in the desired order, by double-clicking in the enemy list in the top-left corner.						|
;																				|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
; Free space usage:																		|
; - Free space used by this patch can be set at the user's will (but within the same respectives banks) ; The default ranges are:				|
;																				|
;  LoROM $88:EE32 to $88:F21A (inclusive) ie PC $04:6E32 to $04:721A (inclusive). [lenght: $ 3E9 bytes]								|
;  LoROM $B8:B7A0 to $B8:B7CB (inclusive) ie PC $1C:37A0 to $1C:37CB (inclusive). [lenght: $  2C bytes]								|
;																				|
; - If there is data in your ROM at these places, you need to relocate the code of this ASM before applying (see below).					|
;																				|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;																				|
; Data modification / Pointer redirection:															|
; - Some data are changed at the folowing ranges in the ROM:													|
;																				|
;  LoROM $88:8235 to $88:823D (inclusive) ie PC $04:0235 to $04:023D (inclusive). [lenght: $   8 bytes]								|
;  LoROM $88:825A to $88:8261 (inclusive) ie PC $04:025A to $04:0261 (inclusive). [lenght: $   8 bytes]								|
;  LoROM $88:827F to $88:8286 (inclusive) ie PC $04:027F to $04:0286 (inclusive). [lenght: $   8 bytes]								|
;  LoROM $88:806E to $88:806F (inclusive) ie PC $04:006E to $04:006F (inclusive). [lenght: $   2 bytes]								|
;  LoROM $88:DB48 to $88:DB5D (inclusive) ie PC $04:5B48 to $04:5B5D (inclusive). [lenght: $  16 bytes]								|
;  LoROM $89:AA08 to $89:AA17 (inclusive) ie PC $04:AA08 to $04:AA1F (inclusive). [lenght: $  18 bytes]								|
;  LoROM $8A:A940 to $8A:B17F (inclusive) ie PC $05:2940 to $05:317F (inclusive). [lenght: $ 840 bytes]								|
;  LoROM $9A:B800 to $9A:B81F (inclusive) ie PC $0D:3800 to $0D:381F (inclusive). [lenght: $  20 bytes] 							|
;  LoROM $9A:B900 to $9A:B95F (inclusive) ie PC $0D:3900 to $0D:395F (inclusive). [lenght: $  60 bytes] 							|
;  LoROM $9A:B980 to $9A:BADF (inclusive) ie PC $0D:3980 to $0D:3ADF (inclusive). [lenght: $ 160 bytes] 							|
;																				|
; - If you have already used these addresses to create HiJack point or other stuff for your hack, you shouldn't apply this patch.				|
;																				|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;																				|
; RAM usage:																			|
; - This patch use several free RAM addresses:															|
;																				|
;  RAM $7E:FA12 to $7E:FA12 (inclusive). [lenght: $ 2 bytes]													|
;  RAM $7E:FA14 to $7E:FA14 (inclusive). [lenght: $ 2 bytes]													|
;  RAM $7E:FA16 to $7E:FA16 (inclusive). [lenght: $ 2 bytes]													|
;  RAM $7E:FA18 to $7E:FA18 (inclusive). [lenght: $ 2 bytes]													|
;  RAM $7E:FA1A to $7E:FA1A (inclusive). [lenght: $ 2 bytes]													|
;  RAM $7E:FA20 to $7E:FA21 (inclusive). [lenght: $ 2 bytes]													|	
;  RAM $7E:FA22 to $7E:FA23 (inclusive). [lenght: $ 2 bytes]													|	
;																				|
; - If another patch/code who use the same addresses is intended to run at the same time than this patch in a room, you need to change the addresses above to	|
;   other unused one (there is plenty : for example, from 7E:FA24 to 7E:FFFF).											|
;																				|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;																				|
; Credit:																			|
; - PJ_Boy for his documentation - specifically his RAM map and the disassembly of bank $88 - and for writing the code of the subroutine that disables parts   	|
;   of BG3 via window 1 mask.																	|
; - Bruce Clark (65C816 Opcodes at http://www.6502.org/tutorials/65c816opcodes.html).										|	
;																				|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;																				|
; Patch Created by BigDomino.																	|
;																				|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*

lorom

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				; Defines addresses :
				; ===================
!OnScreenX	      = $12	; Horizontal coordinate of Samus on-screen.
!windowBg3Mask        = $61	; Window BG3/BG4 mask settings.
!window1Left          = $63	; Window 1 left position.
!window1Right         = $64	; Window 1 right position.
!winAreaSubscrDis     = $6D	; Window area subscreen disable.
!ColMathRegB	      = $71	; Next gameplay colour math control register B.
!BackdropBlue	      = $74	; Value 1 for colour math subscreen backdrop colour (used for Blue).
!BackdropGreen	      = $75	; Value 2 for colour math subscreen backdrop colour (used for Green).
!BackdropRed	      = $76	; Value 3 for colour math subscreen backdrop colour (used for Red).
!PlayerInput	      = $8B	; Controller 1 input.
!FrameCounter	      = $05B8	; Frame counter (incl. lag frames).
!layer1X              = $0911	; Layer 1 X position.
!layer1Y              = $0915	; Layer 1 Y position.
!GameState	      = $0998	; Game state.
!samusX               = $0AF6	; Samus X position.
!samusY               = $0AFA	; Samus Y position.
!PBlendCol1	      = $C032	; Color, copied straight to CGRAM (from palette blend color n1).
!PBlendCol2	      = $C034	; Color, copied straight to CGRAM (from palette blend color n2).
!PBlendCol3	      = $C036	; Color, copied straight to CGRAM (from palette blend color n3).
!PBlendCol1Tar	      = $C232	; Target color (from palette blend color n1).
!PBlendCol2Tar	      = $C234	; Target color (from palette blend color n2).
!PBlendCol3Tar	      = $C236	; Target color (from palette blend color n3).
!bg3X                 = $7ECADC	; BG3 X position.
!bg3Y                 = $7ECADE	; BG3 Y position.
!RAM_Y_Shift          = $7EFA12	; Custom RAM address to store halo vertical shift value.									  [!]
!RAM_X_Shift          = $7EFA14	; Custom RAM address to store halo horizontal shift value.									  [!]
!ColorFlag	      = $FA16	; Custom RAM address for the "color-saved" flag. (#$1234 = set ; #$0000 = clear).			  			  [!]
!OriginalColor	      = $FA18	; Custom RAM address to store palette blend color 2 value, as a reference.					 		  [!]
!DarkerColor	      = $FA1A	; Custom RAM address to store palette blend color 2 value darkened, as a reference.					  	  [!]
!PBOccuredFlag        = $7EFA20	; Custom RAM address for the flag set when a power bomb explosion animation has occured. (#$0001 = set ; #$0000 = clear)	  [!]
!RestartAfterPB       = $7EFA22	; Custom RAM address to store the value of the frame counter where a PB explosion has occured.				  	  [!]
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				; Defines constants :
				; ===================
				;
!spotlightRadius      = #$0040	; Radius of the halo ; BG3 will be amputated, right and left, beyond this distance.
!Shift_positive       = #$0020	; Max value the halo can be shifted in the UP/RIGHT direction. [min. 0000 ; max. 0040]						  [!]
!Shift_negative       = #$FFE0	; Max value the halo can be shifted in the DOWN/LEFT direction.	[min. FFC0 ; max. 0000]					  	  [!]
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
org $89AA08			; Set new palette blend entries N06, 0C, 12 & 18 with 1+3 colors for the halo :
				; ==============================================================================
				; (Each palette overlap with previous palette by one color, as long as color 0 is hardcoded to be always transparent)
				;
				; 	       - Blue  Green  Red    B G R
; Standard dark room (06) :	; ________________________________   _____
;	dw #%????????????????	; Color 0 : #% ? ????? ????? ????? = ? ? ? : outside area  -> use transparency, then do a strong subtraction via backdrop color.
	dw #%0000010000100001	; Color 1 : #% 0 00001 00001 00001 = 1 1 1 : center        -> very low subtraction.
	dw #%0001000010000100	; Color 2 : #% 0 00100 00100 00100 = 4 4 4 : external ring -> medium subtraction.
	dw #%0000100001000010	; Color 3 : #% 0 00010 00010 00010 = 2 2 2 : internal ring -> low subtraction.
				;
; Medium dark room (0C) :	; ________________________________   _____
;	dw #%????????????????	; Color 0 : #% ? ????? ????? ????? = ? ? ? : outside area  -> use transparency, then do a strong subtraction via backdrop color.
	dw #%0000010000100001	; Color 1 : #% 0 00001 00001 00001 = 1 1 1 : center        -> very low subtraction.
	dw #%0001100011000110	; Color 2 : #% 0 00110 00110 00110 = 6 6 6 : external ring -> medium subtraction.
	dw #%0000110001100011	; Color 3 : #% 0 00011 00011 00011 = 3 3 3 : internal ring -> low subtraction.
				;
; Very dark room (12) :		; ________________________________   _____
;	dw #%????????????????	; Color 0 : #% ? ????? ????? ????? = ? ? ? : outside area  -> use transparency, then do a strong subtraction via backdrop color.
	dw #%0000010000100001	; Color 1 : #% 0 00001 00001 00001 = 1 1 1 : center        -> very low subtraction.
	dw #%0010000100001000	; Color 2 : #% 0 01000 01000 01000 = 8 8 8 : external ring -> medium subtraction.
	dw #%0001000010000100	; Color 3 : #% 0 00100 00100 00100 = 4 4 4 : internal ring -> low subtraction.
				;
; Extremely dark room (18) :	; ________________________________   _____
;	dw #%????????????????	; Color 0 : #% ? ????? ????? ????? = ? ? ? : outside area  -> use transparency, then do a strong subtraction via backdrop color.
	dw #%0000010000100001	; Color 1 : #% 0 00001 00001 00001 = 1 1 1 : center        -> very low subtraction.
	dw #%0010110101101011	; Color 2 : #% 0 01011 01011 01011 = B B B : external ring -> medium subtraction.
	dw #%0001100011000110	; Color 3 : #% 0 00110 00110 00110 = 6 6 6 : internal ring -> low subtraction.
				;
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
org $88806E			; Repoint layer blending configuration #$30 :
				; ===========================================
	dw LAYER_BLENDING	; Now point to the routine that set window 1 mask, set color math register, calculate halo glowing effect and backdrop color.
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
org $88DB48			; Halo effect position / animation routine : (hijack the original fog scrolling routine)
				; ==========================================
	PHP 			; Push Processor Statut register to the stack. Always 1 byte copied to the stack.
	PHD			; Push Direct Page register to the stack. Always 2 bytes copied to the stack.
	REP #$30		; Set Accumulator, X and Y index register size to 16 bit (original size is already saved).
	PHA : PHX : PHY		; Push full Accumulator, X and Y register to the stack. 2 bytes if A, X and Y register are in 16 bit mode.
				;
	JSR CHECK_PLAYER_INPUT	; Jump to subroutine that calculate animation of the halo according to input.
	JSR BG3_X_SCROLL	; Jump to subroutine that handle layer 3 and windows mask position. 
				;
	REP #$30		; Set Accumulator, X and Y register size to 16 bit.
	PLY : PLX : PLA		; Pull next tree 2-bytes value from stack toward Y, X register and Accumulator.
	PLD			; Pull next 2-bytes value from stack toward Direct Page register. 
	PLP			; Pull next 2-bytes value from stack toward Processor Statut register.
				;
	PLB			; Pull next 2-bytes value from stack toward Data Bank register (NB: pushed before hijack point).
				;
	RTL			; Return from subroutine long.
				;
;	fillto $88DB8A, $FF	; Fill remaining unused instruction with FF FF FF ... (for the sake of clarity) Uncomment if your assembler know this command.    [!]
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
				; Warn if writing past end of original routine :
	warnpc $88DB8A		; ==============================================
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
org $88EE32        		; Check free space in your ROM at this address ! Code should remain in bank $88.						  [!]
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				;
BG3_X_SCROLL:			; Write BG3 position, store on-screen X position of Samus in $12 :
				; ================================================================
	LDA !samusY		; Load Samus Y position, in pixel, from top border.
	SEC : SBC !layer1Y	; Subtract layer 1 Y position.
	CLC : ADC #$0080	; Shift layer3 position of 80 pixels down to match Samus hitbox center
	EOR #$FFFF : INC A	; Reverse direction of the animation by changing the sign.
	CLC : ADC !RAM_Y_Shift	; Add vertical shift value according to current player inputs.
				;
				; BG3 vertical position need to be limited to prevent the halo to appear at the other side of the screen (-> occure on elevator) :
	CMP #$FE98		; Compare with the minimum value below which the halo start to loop near the ceilling :
	BCC +			; If BG3 position is less than #$FE98, branch +...
	CMP #$FF67+1		; Compare with the maximum value above which the halo start to loop near the floor :
	BCS ++			; If BG3 position is greater than #$FF67 (ie: greater or equal than FF68), branch ++...
	BRA +++			; but if value is between FE98 & FF67 (inclusive), go to the final step to store the calculated value as the new BG3 Y position.
+	LDA #$FE98		; ...block minimal value in accumulator and...
	BRA +++			; ...go to final step to store the value as BG3 Y position.
++	LDA #$FF67		; ...block maximal value in accumulator and...
+++	STA !bg3Y		; ...store as BG3 Y position.
				; 
	LDA !samusX		; Load Samus X position, in pixel, from left border.
	SEC : SBC !layer1X	; Subtract layer 1 X position.
	STA !OnScreenX		; Save on-screen position of Samus.
	CLC : ADC #$0080	; Shift layer3 position of 80 pixels to the right to match Samus hitbox center.
	CLC : ADC !RAM_X_Shift	; Add horizontal shift value according to current player inputs.
	EOR #$FFFF : INC A	; Reverse direction of the animation by changing the sign.
	STA !bg3X		; Store as BG3 X position.
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
				; Window 1 left position = max(0, Samus' X position on screen - spotlight radius +/- halo shift value) :
				; ======================================================================================================
	LDA !OnScreenX		; Load on-screen position of Samus.
				;
	SEC			; 
	SBC !spotlightRadius	; Subtract radius of the halo, in pixel.
	CLC			; 
	ADC !RAM_X_Shift	; Add horizontal shift value according to current player inputs.
				;
	CMP #$00FF		; Compare halo left border current X position with 00FF.
	BCC +			; If less than 00FF (0000 to 00FE) halo is in normal range. Avoid next instruction.
				;
	LDA #$0000		; If equal or greater than 00FF (00FF to FFFF), halo shifted beyond left border (causing overflow). Load 0000 coordinate instead.  
+				;
	SEP #$20		; Set Accumulator size to 8 bit.
	STA !window1Left	; Store low byte of accumulator to window 1 left border RAM address.
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
				; Window 1 right position = min(255, Samus' X position on screen + spotlight radius +/- halo shift value) :
				; =========================================================================================================
	REP #$20		; Set Accumulator size to 16 bit.
				;
	LDA !OnScreenX		; Load on-screen position of Samus.
				;
	CLC			; 
	ADC !spotlightRadius	; Add radius of the halo, in pixel.
	CLC			; 
	ADC !RAM_X_Shift	; Add horizontal shift value according to current player inputs.
				;
	CMP #$0100		; Compare halo right border current X position with 0100.
	BCC +			; If less than 0100 (0000 to 00FF) halo is in normal range. Avoid next instruction.
				;
	LDA #$00FF		; If equal or greater than 0100 (0100 to FFFF), halo shifted beyond right border. Load 00FF coordinate instead. 
+				;
	SEP #$20		; Set Accumulator size to 8 bit.
	STA !window1Right	; Store low byte of accumulator to window 1 right border RAM address.
				;
	RTS			; Return from subroutine.
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
LAYER_BLENDING:			; Save current state of registers : (at $04:6EA9 pc)
				; =================================
	PHB : PHP 		; Push Data bank and Processor Statut register to the stack. Always 1 byte copied to the stack.
	PHD			; Push Direct Page register to the stack. Always 2 bytes copied to the stack.
	REP #$30		; Set Accumulator, X and Y index register size to 16 bit (original size is already saved).
	PHA : PHX : PHY		; Push full Accumulator, X and Y register to the stack. 2 bytes if A, X and Y register are in 16 bit mode.
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
				; Disable halo layer blending routine / colors definition, if power bomb is about to explode :
				; (-> let the power-bomb subroutine handles the colorization of the backdrop layer)
				; ============================================================================================
	LDA $0CEC		; Load PB pre-explosion flash radius pointer.
	CMP #$3400		; Check if value is already $#3400 (= eliptic shaped flash appear on screen).
	BCC +			; If it is not the case, continue to follow the halo's drawing subroutine.
	BRL END_COLOR		; If equal or higher than $#3400, jump to the end of this subroutine.
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
+				; Disable BG3 outside of window 1 :
				; =================================
	SEP #$20		; Set accumulator size to 8 bit.
	LDA #$03		; LDA #% 00000011 ie: Enable exclusive mask on BG3 via window 1.
	STA !windowBg3Mask 	; at RAM address $7E:0061
	LDA #$04		; LDA #% 00000100 ie: Disable BG3 in window area.
	STA !winAreaSubscrDis 	; at RAM address $7E:006D
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
				; Set color math :
				; ================
	LDA #$93 		; LDA 10110011 ie: enable color math on BG1, BG2, sprites and "main screen backdrop".
				; subtract subscreen to main screen.
	STA !ColMathRegB	; Send to color math control registre B.
				;
	LDA #$04		;
	STA $6B    		; Subscreen layers = BG3 (and "subscreen backdrop")
				;
	LDA #$13		;
	STA $69    		; Main screen layers = BG1/BG2/sprite/"main screen backdrop"
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
				; Find BG3 halo external ring's color / Save this color to RAM / Create new target color for glow effect :
				; ========================================================================================================
	LDA #$7E		; Load desired bank byte (7E) into accumulator.
	PHA			; Push that value into the stack.
	PLB			; Pull that value from the stack into the Data Bank Register.
	REP #$30		; Set Accumulator, X & Y registers size to 16 bit.
				;	
	LDA !ColorFlag		; Load current statut of custom color flag.
	CMP #$1234		; If already set...
	BEQ ++			; ...avoid the step that follow, otherwise ...
	LDA !PBlendCol2		; Load A with FX1 blend color n2 from palette (color of the external ring of the halo).
	BNE +			; If $7EC034 is not zero jump to + otherwise...
	LDA !PBlendCol2Tar	; Load A with FX1 blend color n2 from target palette (color of the external ring of the halo).
+				;
	STA !OriginalColor	; Save to custom RAM adress 7E:FA18 for the non-glowing sub-backdrop layer.
	CLC : ADC #$0842	; Create new target color for glow effect by adding 2 shades of grey.
	STA !DarkerColor	; Store target value at custom RAM adress 7E:FA1A.
	LDA #$1234		; Set custom color flag (1234)...
	STA !ColorFlag		; ... at 7E:FA16.
				;
++				;----------------------------------------------------------------------------------------------------------------------------	*
				; 
				; Check if a PB explosion has just finished : If so, wait until the right frame to unfroze the counter and re-enable the glowing :
				; ================================================================================================================================
	LDA !PBOccuredFlag	; Load spcial PB flag to see if an explosion occured.
	CMP #$0001		; if so, continue below...
	BNE ++			; if not, go to ++
	LDA !FrameCounter	; A PB explosion occured : Load frame counter and...
	AND #$00FF		; ... isolate low byte to count from 0 to FF frame.
	CMP !RestartAfterPB	; Check if the counter is on the same frame + 1 than when it was frozen (to continue the glowing animation without error).
	BEQ +			; If this is the correct frame, go to + and...
	BRL BACKDROP:		; if it isn't the correct frame, ignore the glowing subroutine but jump to the subroutine that colors the backdrop layer again.
+				; 
	STZ  !PBOccuredFlag	; ...reset the special PB flag 
++				; 
	LDA !FrameCounter	; Load general frame counter.
	INC			; Calculate value of the next frame...
	AND #$00FF		; Isolate low byte to count from 0 to FF frame.
	STA !RestartAfterPB	; Keep track of the next frame value, in case of PB interruption, to avoid messing up the glowing process.
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
	LDA !FrameCounter	; Load general frame counter.
	AND #$00FF		; Isolate low byte to count from 0 to FF frame.
	TAX			; Copy (save) frame counter value to X index register.
				;
	CMP #$0020		; If frame number is 20...
	BEQ ++			; ... branch ++.
	CMP #$0030		; If frame number is 30...
	BEQ ++			; ... branch ++.
	CMP #$00A0		; If frame number is A0...
	BEQ ++			; ... branch ++.
	CMP #$00B0		; If frame number is B0...
	BEQ ++			; ... branch ++.
				;
	CMP #$0000		; If frame number is 00... (CMP 0000 -> for clarity purpose...)
	BEQ +			; ... branch +.
	CMP #$0010		; If frame number is 10...
	BEQ +			; ... branch +.
	CMP #$0080		; If frame number is 80...
	BEQ +			; ... branch +.
	CMP #$0090		; If frame number is 90...
	BEQ +			; ... branch +.
				;
	BRA +++			; If frame number is none of the above, branch +++.
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
				; Make halo darker (ie: brighter color to subtract) until it reach palette blend original value plus 2 shades :
+				; =============================================================================================================
	LDA !PBlendCol2Tar	; Load A with FX1 blend color n2 from target palette.
	BEQ +++			; If color is set to 0000 (during door transition) jump to the end of routine (ie: do nothing).
	CMP !DarkerColor	; Compare with target glow color.
	BCS +++			; Jump to the end if already reached.
				;
	CLC : ADC #$0421	; Add #$0421 eg. #% 0 00001 00001 00001 = Blue +1 Green +1 Red +1 to make external ring darker.
	STA !PBlendCol2Tar	; Store new color to FX1 blend color n2 from target palette.
	STA !PBlendCol2		; Store new color to FX1 blend color n2 from palette.
				;
	LDA !PBlendCol3Tar	; Load A with FX1 blend color n3 from target palette.
	CLC : ADC #$0421	; Add #$0421 eg. #% 0 00001 00001 00001 = Blue +1 Green +1 Red +1 to make internal ring darker.
	STA !PBlendCol3Tar	; Store new color to FX1 blend color n3 from target palette.
	STA !PBlendCol3		; Store new color to FX1 blend color n3 from palette.
				;
	CPX #$0010		; If counter = 10 darkens center of halo.
	BEQ +			; ...otherwise, continue checking :
	CPX #$0090		; If counter = A0 darkens center of halo.
	BNE +++			; ...otherwise, don't.
+
	LDA !PBlendCol1Tar	; Load A with FX1 blend color n1 from target palette.
	CLC : ADC #$0421	; Add #$0421 eg. #% 0 00001 00001 00001 = Blue +1 Green +1 Red +1 to make the center darker.
	STA !PBlendCol1Tar	; Store new color to FX1 blend color n1 from target palette.
	STA !PBlendCol1		; Store new color to FX1 blend color n1 from palette.
				;
	BRA +++			; Branch to next step (ie: set backdrop color).
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
				; Make halo brighter (ie: darker color to subtract) until it reach back palette blend original value :
++				; ====================================================================================================
	LDA !PBlendCol2Tar	; Load A with FX1 blend color n2 from target palette.
	BEQ +++			; If color is set to 0000 (during door transition) jump to the end of routine (ie: do nothing).
	CMP !OriginalColor	; Compare with target glow color.
	BEQ +++			;
	BCC +++			; Jump to the end if already reached.
				;
	SEC : SBC #$0421	; Subtract #$0421 eg. #% 0 00001 00001 00001 = Blue +1 Green +1 Red +1 to make external ring brighter.
	STA !PBlendCol2Tar	; Store new color to FX1 blend color n2 from target palette.
	STA !PBlendCol2		; Store new color to FX1 blend color n2 from palette.
				;
	LDA !PBlendCol3Tar	; Load A with FX1 blend color n3 from target palette.
	SEC : SBC #$0421	; Subtract #$0421 eg. #% 0 00001 00001 00001 = Blue +1 Green +1 Red +1 to make internal ring brighter.
	STA !PBlendCol3Tar	; Store new color to FX1 blend color n3 from target palette.
	STA !PBlendCol3		; Store new color to FX1 blend color n3 from palette.
				;
	CPX #$0020		; If counter = 20 make center of halo brighter.
	BEQ +			; ...otherwise, continue checking :
	CPX #$00A0		; If counter = B0 make center of halo brighter.
	BNE +++			; ...otherwise, don't.
+
	LDA !PBlendCol1Tar	; Load A with FX1 blend color n1 from target palette.
	SEC : SBC #$0421	; Subtract #$0421 eg. #% 0 00001 00001 00001 = Blue +1 Green +1 Red +1 to make the center brighter.
	STA !PBlendCol1Tar	; Store new color to FX1 blend color n1 from target palette.
	STA !PBlendCol1		; Store new color to FX1 blend color n1 from palette.
+++				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
BACKDROP:			; Calculate correct color of peripheral area given external ring color and transfert to sub-backdrop layer in its 3-bytes format :
				; ================================================================================================================================
	LDX #$0000		; Reset loop counter (see below).
-				;
	LDA !OriginalColor	; Load A with saved FX1 blend color n2 from target palette (color of the external ring of the halo).
				;
	CLC : ADC #$1084	; Add #$1084 eg. #% 0 00100 00100 00100 = Blue +4 Green +4 Red +4 to set external area darker than external ring.
	CPX #$0003		; If counter is 0003 (end)...
	BEQ END_COLOR		; ... jump to the end of this routine.
	CPX #$0000		; If counter is 0000 (start)...
	BEQ +			; ... avoid the two following steps.
	CPX #$0001		; If counter is 0001...
	BEQ ++			; ... avoid the following step.
  LSR : LSR : LSR : LSR : LSR	; Else (counter is 0002), shift 5 bits to the right and...
++				;
  LSR : LSR : LSR : LSR : LSR	; shift 5 bits to the right to get next sub-color.
+				;
	AND #$001F		; AND bitmask : #% 00000000 00011111 to isolate data of ONE sub-color.
				;
	SEP #$20		; Set Accumulator size to 8 bit.
				;
	CPX #$0000		; If counter is 0000 (start)...
	BEQ +			;
	CPX #$0001		; If counter is 0001...
	BEQ ++			;
	CLC : ADC #$80		; Else (counter is 0002) ; Add/set the "Blue" bit  #% 100 00000
	STA !BackdropBlue	; Store as value 1 for colour math subscreen backdrop colour.
	BRA +++			; ... and branch to loop point.
+				;
	CLC : ADC #$20		; Add/set the "Red" bit   #% 001 00000		
	STA !BackdropRed	; Store as value 3 for colour math subscreen backdrop colour.
	BRA +++			; ... and branch to loop point.
++				;
	CLC : ADC #$40		; Add/set the "Green" bit #% 010 00000
	STA !BackdropGreen	; Store as value 2 for colour math subscreen backdrop colour.
+++				;
	INX			; Increment loop counter in X register.
	REP #$20		; Set Accumulator size to 16 bit.
	BRA -			; Loop
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
END_COLOR:			; Restore previous state of registers & RTS :
				; ===========================================
	REP #$30		; Set Accumulator, X and Y register size to 16 bit.
	PLY : PLX : PLA		; Pull next tree 2-bytes value from stack toward Y, X register and Accumulator.
	PLD			; Pull next 2-bytes value from stack toward Direct Page register. 
	PLP : PLB		; Pull next two 2-bytes value from stack toward Processor Statut and Data bank register.
				;
	RTS			; Return from subroutine.
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
CHECK_PLAYER_INPUT:		; Input detection routine :
				; =========================
	REP #$30		; Set accumulator, X & Y register size to 16 bit.
				;
	LDA !PlayerInput	; Controller 1 input ($4218)
	AND #$A00      		; Check if UP+LEFT are pressed.
	CMP #$A00       	;
	BEQ HALO_UP_LE       	; Branch corresponding subroutine
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
	LDA !PlayerInput	; Controller 1 input ($4218)
	AND #$900      		; Check if UP+RIGHT are pressed.
	CMP #$900       	;
	BEQ HALO_UP_RI       	; Branch corresponding subroutine
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
	LDA !PlayerInput	; Controller 1 input ($4218)
	AND #$600      		; Check if DOWN+LEFT are pressed.
	CMP #$600       	;
	BEQ HALO_DO_LE       	; Branch corresponding subroutine
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
	LDA !PlayerInput	; Controller 1 input ($4218)
	AND #$500      		; Check if DOWN+RIGHT are pressed.
	CMP #$500       	;
	BEQ HALO_DO_RI       	; Branch corresponding subroutine
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
	LDA !PlayerInput	; Controller 1 input ($4218)
	AND #$800      		; Check if UP is pressed.
	CMP #$800       	;
	BEQ HALO_UP       	; Branch corresponding subroutine
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
	LDA !PlayerInput	; Controller 1 input ($4218)
	AND #$200      		; Check if LEFT is pressed.
	CMP #$200       	;
	BEQ HALO_LE       	; Branch corresponding subroutine
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
	LDA !PlayerInput	; Controller 1 input ($4218)
	AND #$100      		; Check if RIGHT is pressed.
	CMP #$100       	;
	BEQ HALO_RI       	; Branch corresponding subroutine
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
	LDA !PlayerInput	; Controller 1 input ($4218)
	AND #$400      		; Check if DOWN is pressed.
	CMP #$400       	;
	BEQ HALO_DO       	; Branch corresponding subroutine
				;
				;----------------------------------------------------------------------------------------------------------------------------	*
				;
	BRA HALO_CENTER		; If none of the above conditions are fulfilled, branch HALO_CENTER.
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;
; Halo shifting table :
; =====================
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
HALO_UP_LE:
	JSR INCREASE_Y_STRONG
	JSR DECREASE_X_STRONG
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
HALO_UP_RI:
	JSR INCREASE_Y_STRONG
	JSR INCREASE_X_STRONG
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
HALO_DO_LE:
	JSR DECREASE_Y_STRONG
	JSR DECREASE_X_STRONG
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
HALO_DO_RI:
	JSR DECREASE_Y_STRONG
	JSR INCREASE_X_STRONG
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
HALO_UP:
	JSR INCREASE_Y_STRONG
	JSR CENTER_X_STRONG
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
HALO_LE:
	JSR DECREASE_X_STRONG
	JSR CENTER_Y_STRONG
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
HALO_RI:
	JSR INCREASE_X_STRONG
	JSR CENTER_Y_STRONG
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
HALO_DO:
	JSR DECREASE_Y_STRONG
	JSR CENTER_X_STRONG
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
HALO_CENTER:
	JSR CENTER_Y_LIGHT
	JSR CENTER_X_LIGHT
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;
; Halo shifting calculation :
; ===========================
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
INCREASE_Y_STRONG:
	LDA !RAM_Y_Shift
	CMP !Shift_positive
	BEQ +
	INC A
	CMP !Shift_positive
	BEQ +
	INC A
	CMP !Shift_positive
	BEQ +
	INC A
	STA !RAM_Y_Shift
+
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
DECREASE_Y_STRONG:
	LDA !RAM_Y_Shift
	CMP !Shift_negative
	BEQ +
	DEC A
	CMP !Shift_negative
	BEQ +
	DEC A
	CMP !Shift_negative
	BEQ +
	DEC A
	STA !RAM_Y_Shift
+
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
CENTER_Y_STRONG:
	LDA !RAM_Y_Shift
	BEQ +++
	BMI ++
	DEC A
	BEQ +
	DEC A
	BEQ +
	DEC A
	BEQ +
+
	STA !RAM_Y_Shift
	RTS
++
	INC A
	BEQ +
	INC A
	BEQ +
	INC A
	BEQ +
+
	STA !RAM_Y_Shift
+++
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
CENTER_Y_LIGHT:
	LDA !RAM_Y_Shift
	BEQ +++
	BMI ++
	TAY
	LDA !FrameCounter
	AND #$000F
	TAX
	TYA
	CPX #$0000
	BEQ +
	CPX #$0008
	BEQ +
	BRA +++
+
	DEC A
	STA !RAM_Y_Shift
	RTS
++
	TAY
	LDA !FrameCounter
	AND #$000F
	TAX
	TYA
	CPX #$0000
	BEQ +
	CPX #$0008
	BEQ +
	BRA +++
+
	INC A
	STA !RAM_Y_Shift
+++
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
INCREASE_X_STRONG:
	LDA !RAM_X_Shift
	CMP !Shift_positive
	BEQ +
	INC A
	CMP !Shift_positive
	BEQ +
	INC A
	CMP !Shift_positive
	BEQ +
	INC A
	STA !RAM_X_Shift
+
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
DECREASE_X_STRONG:
	LDA !RAM_X_Shift
	CMP !Shift_negative
	BEQ +
	DEC A
	CMP !Shift_negative
	BEQ +
	DEC A
	CMP !Shift_negative
	BEQ +
	DEC A
	STA !RAM_X_Shift
+
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
CENTER_X_STRONG:
	LDA !RAM_X_Shift
	BEQ +++
	BMI ++
	DEC A
	BEQ +
	DEC A
	BEQ +
	DEC A
	BEQ +
+
	STA !RAM_X_Shift
	RTS
++
	INC A
	BEQ +
	INC A
	BEQ +
	INC A
	BEQ +
+
	STA !RAM_X_Shift
+++
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
CENTER_X_LIGHT:
	LDA !RAM_X_Shift
	BEQ +++
	BMI ++
	TAY
	LDA !FrameCounter
	AND #$000F
	TAX
	TYA
	CPX #$0000
	BEQ +
	CPX #$0008
	BEQ +
	BRA +++
+
	DEC A
	STA !RAM_X_Shift
	RTS
++
	TAY
	LDA !FrameCounter
	AND #$000F
	TAX
	TYA
	CPX #$0000
	BEQ +
	CPX #$0008
	BEQ +
	BRA +++
+
	INC A
	STA !RAM_X_Shift
+++
	RTS

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				; 
PB_CONFIG_Y_0_2_6:		; Alter power-bomb animation's layer blending settings (Y=0/2/6), when an explosion occures in a room with halo effect, to hide BG3 :
				; ===================================================================================================================================
	LDA $196E		; Check FX type pointer.
	CMP #$0C		; If type is not #$0C (halo)...
	BNE +			; go to + to apply the standard settings...
	LDA $0CED		; Load high byte of PB flash radius in 8 bit accumulator to see if a PB explosion occure.
	CMP #$34		; If value is below #$34[00], the animation didn't have started yet...
	BCC +			; go to + to apply the standard settings...
				; otherwise,...
	LDA #$1B		; ...remove BG3 from main screen layers
	STA $69			; at $7E:0069...
	LDA #$00		; ...then, remove BG3 from sub-screen layers
	STA $6B			; at $7E:006B...
	BRA ++			; ...and go to the end of this subroutine.
+				;
	LDA #$13		; Standard settings for main screen layer, that was removed from the PB routine to create hijack point.  
	STA $69			;
	LDA #$04		; Standard settings for sub-screen layer, that was removed from the PB routine to create hijack point.  
	STA $6B			;
++				;
	RTS			; Return to the PB layer blending subroutine.
				; 
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				; 
PB_CONFIG_Y_4:			; Alter power-bomb animation's layer blending settings (Y=4) when an explosion occures in a room with halo effect, to hide BG3 :
				; ==============================================================================================================================
	LDA $196E		; Check FX type pointer.
	CMP #$0C		; If type is not #$0C (halo)...
	BNE +			; go to + to apply the standard settings...
	LDA $0CED		; Load high byte of PB flash radius in 8 bit accumulator to see if a PB explosion occure.
	CMP #$34		; If value is below #$34[00], the animation didn't have started yet...
	BCC +			; go to + to apply the standard settings...
				; otherwise,...
	LDA #$1B		; ...remove BG3 from main screen layers
	STA $69			; at $7E:0069...
	LDA #$00		; ...then, remove BG3 from sub-screen layers
	STA $6B			; at $7E:006B...
	BRA ++			; ...and go to the end of this subroutine.
+				;
	LDA #$11		; Standard settings for main screen layer, that was removed from the PB routine to create hijack point. 
	STA $69			;
	LDA #$06		; Standard settings for sub-screen layer, that was removed from the PB routine to create hijack point.  
	STA $6B			;
++				;
	RTS			; Return to the PB layer blending subroutine.
				; 
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				; 
org $888235			; Hijack point in the power-bomb layer blending subroutine (Y=0/2)
	JSR PB_CONFIG_Y_0_2_6	; Jump to custom subroutine that check FX type in a room. If halo FX is used, hide BG3 during explosion animation.
	NOP : NOP		; Remove original instructions with 5 "No OPeration" opcode.
	NOP : NOP		;
	NOP			;
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				; 
org $88825A			; Hijack point in the power-bomb layer blending subroutine (Y=4)
	JSR PB_CONFIG_Y_4	; Jump to custom subroutine that check FX type in a room. If halo FX is used, hide BG3 during explosion animation.
	NOP : NOP		; Remove original instructions with 5 "No OPeration" opcode.
	NOP : NOP		;
	NOP			;
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				; 
org $88827F			; Hijack point in the power-bomb layer blending subroutine (Y=6)
	JSR PB_CONFIG_Y_0_2_6	; Jump to custom subroutine that check FX type in a room. If halo FX is used, hide BG3 during explosion animation.
	NOP : NOP		; Remove original instructions with 5 "No OPeration" opcode.
	NOP : NOP		;
	NOP			;
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
				; Jump to different custom subroutines :
				; ======================================
org $82F7F0			;
	JSL $B8B7A0		; Jump to the routine at $B8:B7A0 to $B8:B7B5 incl. -> Reset glow color flag during room transition (Spotlight v.1.00.asm).
org $82F7F4			;
				; [RESERVED]						
org $82F7F8			;					  		 \
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F7FC			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F800			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F804			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F808			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F80C			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F810			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F814			;					  		   \ Usefull for further Depth of oblivion dveloppement.
				; Place for other future JSR in bank 82 or JSL anywhere.   / ...Should be removed for your hack.
org $82F818			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F81C			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F820			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F824			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F828			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F82C			;					  		  |
				; Place for other future JSR in bank 82 or JSL anywhere.  |
org $82F830			;					 		  |
				; Place for other future JSR in bank 82 or JSL anywhere. /
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
org $B8B7A0			; Subroutine that reset the Save-color flag during room transition and set the PB Occured flag when a power-bomb explode :
				; ========================================================================================================================
	PHB			; Push Data bank register to the stack.
				;
	PEA $7E00		; Push value 7E00 into the stack.
	PLB : PLB		; Pull high byte ($7E) to set Data Bank Register.
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				;
	LDA !GameState		; Check game state
	AND #$00FF		; Cut high byte.
	CMP #$000B		; Check for door transition.
	BEQ ++			; If door transition occures, ...
				;
	LDA $0CEC		; Load PB pre-explosion flash radius pointer.
	CMP #$3400		; Check if value is already $#3400 (= eliptic shaped flash appear on screen).
	BCS +			; If it is the case jump to +.
	BRA +++			; If not, don't set the PB occured flag and don't reset the color flag but jump to the end of this subroutine.
+				; 
	LDA #$0001		; Load #$0001 value and...
	STA !PBOccuredFlag	; ...set the PB Occured flag.
	BRA +++			; Then, jump to the end of subroutine (without reseting color flag).
++				;
	STZ !ColorFlag		; ...reset (store zeroes) the halo glow color flag.
	STZ !PBOccuredFlag	; ...and reset (store zeroes) the PB Occured flag.
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
+++				;
	PLB			; Pull value from stack toward Data bank registers.
	RTL			; Return from subroutine in custom subroutine array.
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
org $8AA940			; Halo FX3 tilemaps : (From $05:2940 pc to $05:317F pc)
				; ===================
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 00.
				;
	db $61, $18, $61, $18	; * tiles n00 ; n01
	db $61, $18, $61, $18	; * tiles n02 ; n03
	db $61, $18, $61, $18	; * tiles n04 ; n05
	db $61, $18, $61, $18	; * tiles n06 ; n07
	db $61, $18, $61, $18	; * tiles n08 ; n09
	db $61, $18, $61, $18	; * tiles n0A ; n0B
	db $61, $18, $61, $18	; * tiles n0C ; n0D
	db $61, $18, $61, $18	; * tiles n0E ; n0F
	db $61, $18, $61, $18	; * tiles n10 ; n11
	db $61, $18, $61, $18	; * tiles n12 ; n13
	db $61, $18, $61, $18	; * tiles n14 ; n15
	db $61, $18, $61, $18	; * tiles n16 ; n17
	db $61, $18, $61, $18	; * tiles n18 ; n19
	db $61, $18, $61, $18	; * tiles n1A ; n1B
	db $61, $18, $61, $18	; * tiles n1C ; n1D
	db $61, $18, $61, $18	; * tiles n1E ; n1F
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 01.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 02.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 03.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 04.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 05.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 06.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 07.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 08.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 09.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 0A.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $7C, $18, $7D, $18	; 
	db $7E, $18, $7F, $18	; 
	db $7F, $58, $7E, $58	; 
	db $7D, $58, $7C, $58	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 0B.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $75, $18	; 
	db $78, $18, $79, $18	; 
	db $7A, $18, $7B, $18	; 
	db $7B, $58, $7A, $58	; 
	db $79, $58, $78, $58	; 
	db $75, $58, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 0C.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $8D, $18, $70, $18	; 
	db $71, $18, $72, $18	; 
	db $73, $18, $74, $18	; 
	db $74, $58, $73, $58	; 
	db $72, $58, $71, $58	; 
	db $70, $58, $8D, $58	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 0D.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $89, $18	; 
	db $8A, $18, $8B, $18	; 
	db $8C, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $8C, $58	; 
	db $8B, $58, $8A, $58	; 
	db $89, $58, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 0E.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $86, $18	; 
	db $87, $18, $88, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $88, $58, $87, $58	; 
	db $86, $58, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 0F.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $83, $18	; 
	db $84, $18, $85, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $85, $58, $84, $58	; 
	db $83, $58, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 10.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $80, $18	; 
	db $81, $18, $82, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $82, $58, $81, $58	; 
	db $80, $58, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 11.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $80, $98	; 
	db $81, $98, $82, $98	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $82, $D8, $81, $D8	; 
	db $80, $D8, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 12.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $83, $98	; 
	db $84, $98, $85, $98	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $85, $D8, $84, $D8	; 
	db $83, $D8, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 13.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $86, $98	; 
	db $87, $98, $88, $98	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $88, $D8, $87, $D8	; 
	db $86, $D8, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 14.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $89, $98	; 
	db $8A, $98, $8B, $98	; 
	db $8C, $98, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $60, $18	; 
	db $60, $18, $8C, $D8	; 
	db $8B, $D8, $8A, $D8	; 
	db $89, $D8, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 15.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $8D, $98, $70, $98	; 
	db $71, $98, $72, $98	; 
	db $73, $98, $74, $98	; 
	db $74, $D8, $73, $D8	; 
	db $72, $D8, $71, $D8	; 
	db $70, $D8, $8D, $D8	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 16.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $75, $98	; 
	db $78, $98, $79, $98	; 
	db $7A, $98, $7B, $98	; 
	db $7B, $D8, $7A, $D8	; 
	db $79, $D8, $78, $D8	; 
	db $75, $D8, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 17.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $7C, $98, $7D, $98	; 
	db $7E, $98, $7F, $98	; 
	db $7F, $D8, $7E, $D8	; 
	db $7D, $D8, $7C, $D8	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 18.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 19.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 1A.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 1B.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 1C.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 1D.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 1E.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 1F.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;---------------------------------------------------------------------------------------------------------------------------	*
				; FX3 tilemap code line 20.
				;
	db $61, $18, $61, $18	; (tile info ditto line 00)
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
	db $61, $18, $61, $18	; 
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
				; Halo FX3 GFX :
				; ==============
				;---------------------------------------------------------------------------------------------------------------------------	*
org $9AB800			; (From $0D:3800 to $0D:381F pc)
				;
				; Halo 8 x 8 pixel GFX tile n0
	db $FF, $00, $FF, $00	; 
	db $FF, $00, $FF, $00 	; 
	db $FF, $00, $FF, $00 	; 
	db $FF, $00, $FF, $00	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n1
	db $00, $00, $00, $00 	; 
	db $00, $00, $00, $00 	; 
	db $00, $00, $00, $00 	; 
	db $00, $00, $00, $00 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
org $9AB900			; (From $0D:3900 to $0D:395F pc)
				;
				; Halo 8 x 8 pixel GFX tile n2
	db $00, $FF, $00, $FF 	; 
	db $00, $FF, $00, $FF 	; 
	db $01, $FF, $02, $FF 	; 
	db $05, $FF, $0B, $FF 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n3
	db $1B, $FF, $27, $FF 	; 
	db $5F, $FF, $BF, $FF 	; 
	db $7F, $FF, $FF, $FF 	; 
	db $FF, $FE, $FF, $FD 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n4
	db $FF, $FF, $FF, $FF 	; 
	db $FF, $FC, $FF, $F3 	; 
	db $FF, $CC, $FF, $B0 	; 
	db $FF, $40, $FF, $80 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n5
	db $FF, $C7, $FF, $38 	; 
	db $FF, $C0, $FF, $00 	; 
	db $FF, $00, $FF, $00 	; 
	db $FF, $00, $FF, $00 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n6
	db $FF, $C0, $FF, $00 	; 
	db $FF, $00, $FF, $00 	; 
	db $FF, $00, $FF, $00 	; 
	db $FF, $00, $FF, $00 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n7
	db $00, $00, $00, $00 	; 
	db $00, $01, $00, $03 	; 
	db $00, $0F, $00, $1F 	; 
	db $00, $3F, $00, $7F 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
org $9AB980			; (From $0D:3980 to $0D:3ADF pc)
				;
				; Halo 8 x 8 pixel GFX tile n8
	db $00, $3F, $00, $7F 	; 
	db $00, $FF, $00, $FF 	; 
	db $00, $FF, $01, $FF 	; 
	db $02, $FF, $05, $FF 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n9
	db $00, $FF, $01, $FF 	; 
	db $06, $FF, $19, $FF 	; 
	db $67, $FF, $9F, $FF 	; 
	db $7F, $FF, $FF, $FF 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile nA
	db $38, $FF, $C7, $FF 	; 
	db $3F, $FF, $FF, $FF 	; 
	db $FF, $FF, $FF, $FF 	; 
	db $FF, $FF, $FF, $F8 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile nB
	db $7F, $FF, $FF, $FF 	; 
	db $FF, $FF, $FF, $FF 	; 
	db $FF, $FF, $FF, $FF 	; 
	db $FF, $C0, $FF, $3F 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile nC
	db $00, $00, $00, $00 	; 
	db $00, $00, $00, $00 	; 
	db $00, $00, $00, $01 	; 
	db $00, $07, $00, $1F 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile nD
	db $00, $00, $00, $00 	; 
	db $00, $07, $00, $3F 	; 
	db $00, $FF, $00, $FF 	; 
	db $00, $FF, $00, $FF 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile nE
	db $00, $0F, $00, $FF 	; 
	db $00, $FF, $00, $FF 	; 
	db $00, $FF, $00, $FF 	; 
	db $00, $FF, $07, $FF 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile nF
	db $00, $FF, $00, $FF 	; 
	db $00, $FF, $00, $FF 	; 
	db $00, $FF, $00, $FF 	; 
	db $7F, $FF, $80, $FF 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n10
	db $01, $FF, $02, $FF 	; 
	db $02, $FF, $02, $FF 	; 
	db $02, $FF, $02, $FF 	; 
	db $02, $FF, $02, $FF 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n11
	db $7F, $FE, $FF, $FE 	; 
	db $FF, $FD, $FF, $FD 	; 
	db $FF, $FD, $FF, $FD 	; 
	db $FF, $FD, $FF, $FD 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n12
	db $FF, $80, $FF, $80 	; 
	db $FF, $00, $FF, $00 	; 
	db $FF, $00, $FF, $00 	; 
	db $FF, $00, $FF, $00 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n13
	db $00, $7F, $00, $7F 	; 
	db $00, $7F, $00, $7F 	; 
	db $00, $FF, $01, $FF 	; 
	db $01, $FF, $01, $FF 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n14
	db $5F, $FF, $5F, $FF 	; 
	db $BF, $FF, $BF, $FF 	; 
	db $BF, $FF, $7F, $FE 	; 
	db $7F, $FE, $7F, $FE 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n15
	db $FF, $A0, $FF, $A0 	; 
	db $FF, $40, $FF, $40 	; 
	db $FF, $40, $FF, $80 	; 
	db $FF, $80, $FF, $80 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n16
	db $00, $0F, $00, $0F 	; 
	db $00, $1F, $00, $1F 	; 
	db $00, $1F, $00, $3F 	; 
	db $00, $3F, $00, $3F 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n17
	db $05, $FF, $0B, $FF 	; 
	db $0B, $FF, $17, $FF 	; 
	db $17, $FF, $2F, $FF 	; 
	db $2F, $FF, $5F, $FF 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n18
	db $FF, $FD, $FF, $FA 	; 
	db $FF, $F4, $FF, $F4 	; 
	db $FF, $E8, $FF, $E8 	; 
	db $FF, $D0, $FF, $D0 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n19
	db $00, $00, $00, $00 	; 
	db $00, $00, $00, $01 	; 
	db $00, $01, $00, $03 	; 
	db $00, $03, $00, $07 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n1A
	db $00, $3F, $00, $7F 	; 
	db $00, $FF, $00, $FF 	; 
	db $00, $FF, $01, $FF 	; 
	db $02, $FF, $05, $FF 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n1B
	db $17, $FF, $2F, $FF 	; 
	db $5F, $FF, $BF, $FF 	; 
	db $BF, $FF, $7F, $FF 	; 
	db $FF, $FE, $FF, $FD 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n1C
	db $FF, $FA, $FF, $F4 	; 
	db $FF, $E8, $FF, $D0 	; 
	db $FF, $A0, $FF, $40 	; 
	db $FF, $80, $FF, $00 	; 
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				; Halo 8 x 8 pixel GFX tile n1D
	db $00, $00, $00, $01 	; 
	db $00, $03, $00, $07 	; 
	db $00, $0F, $00, $0F 	; 
	db $00, $1F, $00, $3F 	; 
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*


