;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;																				|
; SUBROUTINE ARRAY ASM  [Filename: Subroutine_Array_v.1.00.asm] (version 1.00 ; September 2019)									|
; ====================																		|
;																				|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;																				|
; Description:																			|
; - This patch hijack the main-game loop in bank $82 and jump to a place from which several custom subroutines can be launched without affecting the		|
;   current registers values / states.																|
; - By itself, this patch do nothing noticable during gameplay.													|
; - This patch need to be applied prior to some BigDomino's patches.												|
;																				|
; Warning:																			|
; - This patch use free space at different places of the ROM. Be sure to check the "Free space usage" section before applying.					|
;																				|
; How to install ?:																		|
; - Control free space usage (see below).															|
; - Apply this ASM patch.																	|
; - Done.																			|
;																				|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;																				|
; Free space usage:																		|
; - Free space used by this patch can be set at the user's will ; The default ranges are:									|
;																				|
;  LoROM $82:F7E0 to $82:F83C (inclusive) ie PC $01:77E0 to $01:783C (inclusive). [lenght: $  5D bytes] 							|
;																				|
; - If there is data in your ROM at these places, you need to relocate the code of this ASM before applying (see below).					|
;																				|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;																				|
; Data modification / Pointer redirection:															|
; - Some data are changed at the folowing ranges in the ROM:													|
;																				|
;  LoROM $82:895D to $82:895F (inclusive) ie PC $01:095D to $01:095F (inclusive). [lenght: $   3 bytes]								|
;																				|
; - If you have already used these addresses to create HiJack point or other stuff for your hack, you shouldn't apply this patch.				|
;																				|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
;																				|
; Patch Created by BigDomino.																	|
;																				|
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*

lorom

;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
org $82895D			; HIJACK POINT IN MAIN-GAME LOOP :
				; ================================
	JSR $F7E0		; Jump to custom instructions array in bank $82.
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
org $82F7E0        		; PLACE DEVOTED TO LAUNCH MANY FUTURES SUBROUTINES :
				; ==================================================
				; (Check free space in your ROM at this address ! Should remain in bank $82).							  [!]
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				;
	STZ $071F		; Original instruction removed by hijack point.
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				;
				; Save current state of registers and set DBR :
				; =============================================
	PHB : PHP		; Push Data bank and Processor Statut register to the stack. Always 1 byte copied to the stack.
	PHD			; Push Direct Page register to the stack. Always 2 bytes copied to the stack.
	REP #$30		; Set Accumulator, X and Y index register size to 16 bit (original size is already saved).
	PHA : PHX : PHY		; Push full Accumulator, X and Y register to the stack. 2 bytes if A, X and Y register are in 16 bit mode.				
				;
	PEA $8200		; Push value 8200 into the stack.
	PLB : PLB		; Pull high byte ($82) to set Data Bank Register.
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				;
				; Jump to different custom subroutines :
				; =============================================
org $82F7F0			;
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere. \
org $82F7F4			;						    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F7F8			;						    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F7FC			;					  	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F800			;					  	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F804			;					  	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F808			;					  	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F80C			;					  	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F810			;					  	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F814			;					  	     \ For further Depth of oblivion developpement.
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.   / 
org $82F818			;					  	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F81C			;					  	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F820			;					  	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F824			;					 	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F828			;					 	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F82C			;					 	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere.  |
org $82F830			;					 	    |
	NOP : NOP : NOP	: NOP	; Place for future JSR in bank 82 or JSL anywhere. /
				;
				;---------------------------------------------------------------------------------------------------------------------------	*
				;
org $82F834			; Restore previous state of registers & RTS :
				; ===========================================
	REP #$30		; Set Accumulator, X and Y register size to 16 bit.
	PLY : PLX : PLA		; Pull next tree 2-bytes value from stack toward Y, X register and Accumulator.
	PLD			; Pull next 2-bytes value from stack toward Direct Page register. 
	PLP : PLB		; Pull next 2-bytes value from stack toward Processor Statut and Data bank registers.
				;
	RTS			; RTS, in main game loop, after hijack point.
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*
				;
				; Warn if writing past end of bank : [Write the first byte of the next bank, ie, for a code in $82, $83:8000]
	warnpc $838000		; ==================================
				;
;------------------------------------------------------------------------------------------------------------------------------------------------------------	*