; Palette FX for your beam, by H A M
; Makes all of the beam palettes be able to be animated without using a palette FX object slot.
; Fixes some vanilla bugs:
;     Unpausing or having hyper beam enabled while crystal flashing corrupts the crystal flash palette for a short time
;     Unpausing with hyper beam enabled makes the beam palette have the non-hyper beam palette shortly before being replaced by hyper beam palette
;     Grappling with hyper beam enabled makes the flare flash rainbow
; Optionally moves the grapple beam palette line to sprite palette 6 (beam palette) from 5 (color 15h),
; allowing it to be more customizable and also have an animated palette. Grapple beam graphics can be edited by editing "grapple tiles.gfx".
; 1.01: palette updates properly when picking up a beam upgrade
lorom

!BeamGlowPointerTable = $9BCBFB ; put them in some freespace below bank $C0

!BeamPalettePointer = $0DC8 ; put them in some free RAM from $0000..1FFF (see https://patrickjohnston.org/ASM/Lists/Super%20Metroid/RAM%20map.asm), uses 2 bytes each
!BeamGlowTimer = $0DCA
!BeamGlowBasePointer = $0DCC

!GrappleFlarePaletteMode = 0 ; Set to 1 (vanilla) for a unique grapple flare palette, set to 0 for it to take the current beam palette.
!GrapplePaletteLineChangeAndGFX = 1 ; Patch grapple beam palette line and insert new graphics. Cannot be reversed.

org $90C3D1 ; overwrite vanilla beam palettes (+ space for additional beam tiles pointers for splasma)
HandleBeamGlow:
{
BIT $1E79 : BPL .Return ; If palette FX objects disabled: return (these get disabled when in door transition, X-raying or dying)
LDA $0ACC : CMP #$0007 : BEQ .Return ; If [special Samus palette type] = 7 (crystal flash): return (so it won't overwrite the crystal flash palette)
DEC !BeamGlowTimer : BNE .Return ; If [beam glow timer--] != 0: return
LDY !BeamPalettePointer ; Y = [beam palette pointer]
PHB : PEA.w !BeamGlowPointerTable>>8 : PLB : PLB ; Push [DB], DB = [beam glow pointer table bank]
LDA $0000,y : BNE .NotLoop ; If [[Y]] = 0:
LDY !BeamGlowBasePointer ; Y = beam palette pointer = [beam glow base pointer]
LDA $0000,y
.NotLoop
STA !BeamGlowTimer ; Beam glow timer = [[Y]]
TDC : TAX ; X = 0
INY #2 ; Y += 2
- LDA $0000,y : STA $7EC1C2,x : INX #2 : INY #2 : CPX #$001E : BNE - ; Write beam palette
PLB ; Pull [DB]
STY !BeamPalettePointer ; Beam palette pointer += 20h
.Return
RTL
}

UpdateBeamGlowIfNeeded: ; to prevent resetting the glow if the beam color is the same after unpausing
{
LDA $09A6 : AND #$000F : ASL : TAX : LDA !BeamGlowPointerTable,x : TAX ; Load normal beam glow
LDA $0A76 : BEQ .NoHyper ; If hyper beam enabled:
LDX #Hyper ; Load hyper beam glow
.NoHyper
if !GrappleFlarePaletteMode == 1
	LDA $0D32 : CMP #$C4F0 : BEQ .NoGrapple ; If [grapple beam function] != inactive:
	LDX #Grapple ; Load grapple beam flare glow
	.NoGrapple
endif
CPX !BeamGlowBasePointer : BEQ HandleBeamGlow_Return ; If [X] = [beam glow base pointer]: return
STX !BeamPalettePointer : STX !BeamGlowBasePointer ; Beam palette pointer = beam glow base pointer = [X]
TDC : INC : STA !BeamGlowTimer ; Beam glow timer = 1
JML HandleBeamGlow ; Update palette
}

org !BeamGlowPointerTable
{
; Beam glow format:
; dw $tttt,$pppp,$pppp...
;    $tttt,$pppp,$pppp...
;    ...
;    $0000 ; loop back to beginning
; Where:
;    t = number of frames the palette appears
;    p = 15-color palette
dw Power,Wave,Ice,IceWave,
   Spazer,WaveSpazer,IceSpazer,IceWaveSpazer,
   Plasma,WavePlasma,IcePlasma,IceWavePlasma,
   SpazerPlasma,WaveSpazerPlasma,IceSpazerPlasma,IceWaveSpazerPlasma
Power:
dw $FFFF,$7FFF,$19FF,$1D55,$10AD,$53FF,$039E,$295F,$18DF,$0000,$0000,$0000,$0000,$0000,$0000,$104A, ;normal
   $0000
; dw $FFFF,$7FFF,$7EC0,$6DE0,$54E0,$7F6E,$730A,$6665,$5E22,$7FA0,$7B60,$7720,$72A0,$6E60,$7F91,$20C0,   ;grapple ice
   ; $0000
Wave:
;dw $FFFF,$7FFF,$19FF,$1D55,$10AD,$53FF,$039E,$295F,$18DF,$0000,$0000,$0000,$0000,$0000,$0000,$104A, ;normal
   ;$0000
dw $FFFF,$7FFF,$7C1F,$5816,$300C,$7EDF,$7DDF,$7CDF,$681A,$0000,$0000,$0000,$0000,$0000,$0000,$2409,
   $0000
WaveSpazer:
; dw $FFFF,$7FFF,$7C1F,$5816,$300C,$7EDF,$7DDF,$7CDF,$681A,$0000,$0000,$0000,$0000,$0000,$0000,$2409,
   ; $0000
dw $FFFF,$7FFF,$3BE0,$2680,$1580,$6BFA,$4BF2,$2BEA,$03E0,$0000,$0000,$0000,$0000,$0000,$0000,$0920,
   $0000
Ice:
IceWave:
IceSpazer:
IceWaveSpazer:
IcePlasma:
IceWavePlasma:
IceSpazerPlasma:
IceWaveSpazerPlasma:
Grapple:
dw $FFFF,$7FFF,$7EC0,$6DE0,$54E0,$7F6E,$730A,$6665,$5E22,$7FA0,$7B60,$7720,$72A0,$6E60,$7F91,$20C0,
   $0000
Spazer:
dw $FFFF,$7FFF,$3BE0,$2680,$1580,$6BFA,$4BF2,$2BEA,$03E0,$0000,$0000,$0000,$0000,$0000,$0000,$0920,
   $0000
; dw $FFFF,$7FFF,$03FF,$0216,$00EE,$6BFF,$4BFF,$2BFF,$037B,$0000,$0000,$0000,$0000,$0000,$0000,$00AD,
   ; $0000
Plasma:
WavePlasma:
SpazerPlasma:
WaveSpazerPlasma:
dw $FFFF,$7FFF,$3BE0,$2680,$1580,$6BFA,$4BF2,$2BEA,$03E0,$0000,$0000,$0000,$0000,$0000,$0000,$0920,
   $0000
Hyper: ; Vanilla hyper beam doesn't set the last color for some reason (barely noticable) so I copy the last plasma beam color for the remaining palettes...
dw $0002,$7FFF,$661F,$351F,$001F,$673F,$4E7F,$319F,$18DF,$0000,$0000,$0000,$0000,$0000,$0000,$0920,
   $0002,$7FFF,$36BD,$1D9B,$00D9,$6B9F,$4F3F,$2EDF,$0E7F,$0000,$0000,$0000,$0000,$0000,$0000,$0920,
   $0002,$7FFF,$033B,$0216,$0173,$6BFF,$4BFF,$2BFF,$03FF,$0000,$0000,$0000,$0000,$0000,$0000,$0920,
   $0002,$7FFF,$1F8E,$164B,$0D8A,$6BFD,$4BF9,$2BF5,$03F0,$0000,$0000,$0000,$0000,$0000,$0000,$0920,
   $0002,$7FFF,$3BE0,$2680,$1580,$6BFA,$4BF2,$2BEA,$03E0,$0000,$0000,$0000,$0000,$0000,$0000,$0920,
   $0002,$7FFF,$5F60,$4A40,$3540,$77B4,$5F8E,$4B28,$3301,$0000,$0000,$0000,$0000,$0000,$0000,$0920,
   $0002,$7FFF,$7EC0,$6DE0,$54E0,$7F6E,$730A,$6665,$5E22,$0000,$0000,$0000,$0000,$0000,$0000,$20C0,
   $0002,$7FFF,$7D70,$650B,$4486,$7F37,$7A75,$71B2,$652E,$0000,$0000,$0000,$0000,$0000,$0000,$0920,
   $0002,$7FFF,$7C1F,$5816,$300C,$7EDF,$7DDF,$7CDF,$681A,$0000,$0000,$0000,$0000,$0000,$0000,$2409,
   $0002,$7FFF,$711F,$489B,$1816,$731F,$663F,$593F,$407D,$0000,$0000,$0000,$0000,$0000,$0000,$0920,
   $0000
}

org $90AC8D ; so hyper beam can load properly without plasma beam and it doesn't load beam palette unlike vanilla
{
UpdateBeamTiles:
PHB : PHK : PLB
LDX $0330
LDA #$0100 : STA $D0,x ; size
LDA $0A76 : BEQ .NoHyper ; use plasma beam graphics for hyper beam
LDA #$F800 : BRA +
.NoHyper
LDA $09A6 : AND #$000F : ASL : TAY : LDA $C3B1,y ; normal beam graphics
+ STA $D2,x ; source address
LDA #$009A : STA $D4,x ; source bank
LDA #$6300 : STA $D5,x ; destination address
TXA : CLC : ADC #$0007 : STA $0330
PLB : RTL
HandlePaletteFXObjectsAndBeamGlow:
JSL $8DC527 : JML HandleBeamGlow
LoadBeamTilesAndPalette:
JSL UpdateBeamTiles
UpdateBeamGlow: ; force reset the beam glow even if the type of glow is the same
STZ !BeamGlowBasePointer ; Beam glow base pointer = 0 (force an update)
JML UpdateBeamGlowIfNeeded
LoadBeamTilesAndPaletteIfNeeded:
JSL UpdateBeamTiles : JML UpdateBeamGlowIfNeeded
}

; hijacks and stuff
org $91E128 : JSL LoadBeamTilesAndPalette ; when initializing Samus
org $918829 : JSL LoadBeamTilesAndPalette ; when starting demo
org $80A0C2 : JSL LoadBeamTilesAndPalette ; when starting game
org $82E4CC : JSL LoadBeamTilesAndPalette ; when in door transitions
; org $91E389 : JSL LoadBeamTilesAndPalette ; in a debug function (commented it out because it's unused)
org $91E5F6 : JSL LoadBeamTilesAndPalette : BRA $05 ; when enabling hyper beam in the Mother Brain fight (also don't spawn hyper beam palette FX object)
org $82A377 : JSL UpdateBeamTiles : JSL UpdateBeamGlowIfNeeded : RTS ; when unpausing
org $82A303 : REP #$30 : JSR $A377 : JSR $A380 ; to not crash when doing the above
org $91DBEE : JSL UpdateBeamGlow ; after crystal flash
org $8488D8 : JSL LoadBeamTilesAndPaletteIfNeeded ; when picking up beam

org $90ADA9 : PLP : RTL ; don't spawn hyper beam palette FX object in door transitions or elevators (idk why using an elevator kills all projectiles onscreen)

org $828B54 : JSL HandlePaletteFXObjectsAndBeamGlow

; need to re-order stuff to load grapple palette correctly
org $9BC676 : LDA #$C703 : STA $0D32 ; starting grapple beam
org $9BC682 : JSL UpdateBeamGlowIfNeeded
org $9BC68D : LDA #$EB86 : STA $0A5C
org $9BC8A2 : LDA #$C4F0 : STA $0D32 : JSL UpdateBeamGlowIfNeeded : BRA $01 ; ending grapple beam
org $9BC96A : JMP $C875 ; dedupe code to free some space
org $9BCA15 : JMP $C875
org $9BCBAB : JMP $C875
org $90F4F2 : LDA #$C4F0 : STA $0D32 : JSL UpdateBeamGlowIfNeeded : BRA $01
org $91E68D : BRA + : org $91E694 : + ; let the routine at $82A377 take over when unpausing while grappling

if !GrapplePaletteLineChangeAndGFX == 1
{
	org $9BC686 : BRA $05 ; skip setting sprite palette 5 colour Fh to (11h,1Ch,1Fh)
	
	org $94B18B ; modify instruction list
	dw $0005,$3C21,
	   $0005,$3C22,
	   $0005,$3C23,
	   $0005,$3C24,
	   $B0F4,$B18B  ; Go to $B18B
	
	org $94B13C : LDA #$3C20 : org $94B17C : LDA #$3C20 ; grapple beam end
	
	org $9A8200 : incbin "grapple tiles.gfx" ; import new grapple tiles that fit in a single row
	
	; rewrite updating grapple beam tiles routine to make it work with the more compact graphics
	org $9BBFBD
	; grapple beam end animation
	DEC $0D3E : BPL +
	LDA #$0005 : STA $0D3E
	LDA $0D40 : CLC : ADC #$0020 : CMP #$8280 : BMI ++
	LDA #$8200
	++ STA $0D40
	; grapple beam end tile
	+ LDX $0330
	LDA #$0020 : STA $D0,x ; size
	LDA $0D40 : STA $D2,x : LDA #$009A : STA $D4,x ; source address
	LDA #$6200 : STA $D5,x ; destination address
	TXA : CLC : ADC #$0007 : STA $0330
	; grapple beam tiles
	LDA $0CFB : LSR : AND #$003E : TAY
	LDX $0330
	LDA #$0080 : STA $D0,x
	LDA $C342,y : STA $D2,x : LDA #$009A : STA $D4,x
	LDA #$6210 : STA $D5,x
	TXA : CLC : ADC #$0007 : STA $0330
	RTS

	org $9BC342
	dw $8380,$8380,$8380, $8300,$8300,$8300,$8300,$8300,$8300,$8300,$8300, $8280,$8280,$8280,$8280,$8280,
	   $8280,$8280,$8280,$8280,$8280, $8300,$8300,$8300,$8300,$8300,$8300,$8300,$8300, $8380,$8380,$8380
}
endif
