;----------------BOSS AREA EVENT SYSTEM-----------------
;------------Original Hex Edits by PHOSPHOTiDYL---------
;---------------Modifications by OmegaDragnet9----------

;Credits to P.JBoy for the Banklogs, which are referenced heavily. 

;-------------------------------------------------------
;--------IN VANILLA, YOU ALREADY SET EVENTS-------------
;---------------WHEN YOU KILL A BOSS--------------------
;-------------------------------------------------------

;A boss requires an area-specific bit.
;This is to spawn the boss, and to keep the boss from coming back. 
;The disadvantage is it requires hardcoding to move to another area.

;It's also easy to overlook that
;this already ties up part of the usable events from $16 to $FF.
;(Specifically Events $40-$7F.)

;(You could unintentionally set a boss as defeated using an Event PLM.)

;The specific events are marked in the defines section AND 
;the Boss Bits Array section for reference and ease of use. 

;-------------------------------------------------------
;----------WHAT THIS RESOURCE CHANGES-------------------
;-------------------------------------------------------

;Bosses are now spawned based off whatever 
;room they happen to be in (via the room index number.)

;(This functionality is intended to work in conjunction 
;with Event Gray Doors.) 

;It is possible to have multiple iterations of the same boss.
;You can have each boss work in any area.

;-------------------------------------------------------
;---------A WORD ABOUT THE BOSS MAP ICONS---------------
;-------------------------------------------------------

;In order to use the Boss Map icons,
;consult the Boss Event Bit Array (below). 
;You have to set one of the 8 Boss event bits
;for a given area.

;If you are in Brinstar and set a Norfair event,
;the boss icon will NOT become a red X on the Brinstar map.
;In this capacity you are limited to exactly the 8 events.

;This does not limit the number of boss icons you 
;can technically use in a given map,
;nor does it limit the number of bosses in the area.
;But using the Vanilla Boss icon routine, each map can only unmark
;eight Boss icons, and they MUST be one of the eight events for the area.

;-------------------------------------------------------
;-----------------BOSS BITS-----------------------------
;-------------------------------------------------------

	;RAM
	;ADDRESS     EVENTS
	
	; D828		:40-42	Crateria, only 42 used		  (Bomb Torizo)
				;43-47	Unused
	
	; D829		:48-4A	Brinstar, only 48 and 49 used (Kraid and SporeSpawn)
				;4B-4F	Unused
	
	; D82A		:50-52	Norfair, only 50 and 52 used  (Ridley and Gold Torizo)
				;53-57	Unused
	
	; D82B		:58-5A 	Wrecked Ship, only 58 used	  (Phantoon)
				;5B-5F	Unused
	
	; D82C		:60-62	Maridia, only 60 and 61 used  (Draygon and Botwoon)
				;63-67 	Unused
	
	; D82D		:68-6A	Tourian, only 69 used		  (Mother Brain)
				;6B-6F	Unused
	
	; D82E		:70-72 	Ceres, only 70 used?		  (Ceres Ridley)
				;73-77 	Unused
	
	; D82F		:78-7A	Debug, nothing used in Vanilla
				;7B-7F	Unused

;-------------------------------------------------------
;--------------THE GOLDEN FOUR--------------------------
;-------------------------------------------------------

;In order to use G4, you must set the Room Index to the exact
;events specified in the Defines section.

;At least one Kraid fight needs to have a Room Index of 48
;to turn the Kraid statue gray. 

 lorom

;-------------------------------------------------------
;----------------DEFINES--------------------------------
;-------------------------------------------------------

!RoomIndex  = $079D

!BombTorizo = #$0042
!GoldTorizo = #$0052

!SporeSpawn = #$0048
!Crocomire  = #$0051
!Botwoon    = #$0061

!Kraid      = #$0048
!Phantoon   = #$0058
!Draygon    = #$0060
!Ridley     = #$0050

!BombTorizo = #$0042
!GoldTorizo = #$0052

!PhantoonStatue = #$0006
!RidleyStatue   = #$0007
!DraygonStatue  = #$0008
!KraidStatue    = #$0009

;-------------------------------------------------------
;---------------BOTWOON---------------------------------
;-------------------------------------------------------

org $B39583		;AI Initializing. 

LDA !RoomIndex		
JSL $808233		;Check against Event A	
BCC $24

;----------Botwoon Set as Dead---------------------------

org $B39B13 

LDA !RoomIndex	
JSL $8081FA		;Sets the event based off A (room index.)

;------------------------------------------------------
;---------------SPORESPAWN-----------------------------
;------------------------------------------------------

org $A5EA8F 	;AI Initialization

NOP #3
LDA !RoomIndex	
JSL $808233		;Check against Event A	          
BCC $24

;----------SporeSpawn Set as Dead---------------------------

org $A5EE30

LDA !RoomIndex	
JSL $8081FA		;Sets the event based off A (room index.)
JMP $EE3E

;------------------------------------------------------
;---------------CROCOMIRE------------------------------
;------------------------------------------------------

org $A48A73		;AI Initialization

NOP #3
LDA !RoomIndex		
JSL $808233		;Check against Event A	
BCS $5D


org $A4F67A		;Crocomire Tongue AI Initialization

NOP #3
LDA !RoomIndex	
JSL $808233		;Check against Event A	
BCS $25


;----------Crocomire Set as Dead-------------------------

org $A49B90    

LDA !RoomIndex	
JSL $8081FA		;Sets the event based off A (room index.)
JMP $9B9E		

;------------------------------------------------------
;---------------DRAYGON--------------------------------
;------------------------------------------------------

org $A592D4 			;Draygon set as dead. 

LDA !RoomIndex	
JSL $8081FA		;Sets the event based off A (room index.)
JMP $92E2				 		

;------------------------------------------------------
;---------------KRAID----------------------------------
;------------------------------------------------------

org $A7A92D

NOP #3
LDA !RoomIndex		
JSL $808233		;Check against Event A	
BCC $05


org $A7C822

NOP #3
LDA !RoomIndex		
JSL $808233		;Check against Event A	
BCS $0E


org $A7C82E

LDA !RoomIndex	
JSL $8081FA		;Sets the event based off A (room index.)

;------------------------------------------------------
;---------------PHANTOON-------------------------------
;------------------------------------------------------

org $A7DB7B

LDA !RoomIndex	
JSL $8081FA		;Sets the event based off A (room index.)
NOP #7	

;Wrecked Ship and Phantoon Specific. Only practical for it to be hardcoded. 

org $948E83 ;Region Number

LDA $079F
CMP #$0003
BNE $09 

org $948E8B ;Checks Boss Bit for spike blocks in wrecked ship

LDA !Phantoon
JSL $808233	
BCC $3A

org $8781BA  ; Treadmills

LDA !Phantoon
JSL $808233	
BCS $07

;------------------------------------------------------
;---------------Torizos--------------------------------
;------------------------------------------------------

;Intended to work with Nodever2's Torizo Speed ASM.

org $AAC87F ;If Area Torizo is Dead

LDA !RoomIndex		
JSL $808233
BCC $0D

org $AAB24D

LDA !RoomIndex	
JSL $8081FA		;Sets the event based off A (room index.)

;--------------BT Gray Haze Removal----------------------

; org $AAC90F
; db $6B ;originally 22

;-------------------BT Placement-------------------------

; org $AAC8E0
; NOP #3

; org $AAC8E6
; NOP #3 

;------------------------------------------------------
;---------------RIDLEY---------------------------------
;------------------------------------------------------

org $A6A0F5		;Check if Ridley is dead

NOP #3
LDA !RoomIndex		
JSL $808233
BCC $0D
 
org $A6C5DF		

LDA !RoomIndex	
JSL $8081FA		;Sets the event based off A (room index.)

org $A6A369		;Checks for Norfair's Region Number 
;Not certain what this is needed for yet. 
LDY $079F
CPY #$0002
BNE $03 

;------------------------------------------------
;----------Other Norfair Checks------------------
;------------------------------------------------

;This next section is from 
;PHOSPHOTiDYL's Ridley Everywhere resource.
;Credit goes to PHOSPHOTiDYL for discovering 
;where to NOP and change all the branches to BRA.

org $A6A15C ;Hiding Ridley before fight begins.

	NOP #6 : BRA $03

org $A6A424 ;Music, determined by area

	NOP #6 : BRA $06

;org $A6A44D ;;The Music Itself

;LDA #$0005 : JSL $808FC1 : RTS

org $A6A469 ;Ridley Roar

	NOP #6 : BRA $06

org $A6A478	;BG Fade in and Lava Rising

	NOP #8

org $A6D914 ;Causes Room shaking, only happens in Ceres. 
;Unreachable code for Norfair. 

	NOP #6 : BRA $38

org $A6D93B ;Screen Shake

	NOP #6 : BRA $05

org $A6DF8A ;Ridley is hit. 

	NOP #6 : BRA $1A

org $A6E4D2

	NOP #6 : BRA $3A

;------------------------------------------------
;----------Golden Four---------------------------
;------------------------------------------------

;org $878303 ;go to y+2 if any of the boss bits per area are set.

;org $88DBD7 ;Tourian Statue Preinstruction

;PHP
;REP #$30
;LDA !PhantoonStatue
;JSL $808233
;BCC $3B
;LDA !RidleyStatue
;JSL $808233
;BCC $32
;LDA !DraygonStatue 
;JSL $808233
;BCC $29
;LDA !KraidStatue
;JSL $808233
;BCC $20



