;-----------------------------------------------------------------------------------------
;--------------------Event Change PLM Plus Maridia Tube-----------------------------------
;-----------------------------------------------------------------------------------------
;----------------Based off Dewhi100's Better Room Clear PLM-------------------------------
;-----------------------------------------------------------------------------------------
;---------------------------Version 2.1---------------------------------------------------

;----------------------------IN EDITOR----------------------------------------------------

;High Byte: Set to zero for regular event triggering.
;Set to "4000" for event unsetting.

;Low Byte: Set to the Event to be triggered or unset.

;-----------------------------FEATURES----------------------------------------------------

;Implements earthquake (as featured in MetroidNerd#9001 and Benox50's
;versions of the Event PLM.)
;Implements an event unset feature based off the High byte of the PLM Value.
;Optional earthquake from Room ASM.
;Turns the Maridia Tube PLM into its own Event PLM

;The earthquakes will trigger if you enter into the Room ASM the 
;last 4 digits of !8FRoomShake in the editor.

;Repoint freespace as needed.

;To trigger PLM and/or Earthquake immediately, just set the Enemies Needed
;value to zero in the editor.

;---------------------------CREDITS------------------------------------------------------

;Dewhi100 for his Better Room Clear PLM.
;MetroidNerd#9001 for earthquake, 
;Benox50 for event unset
;Amoeba and TestRunner (because I compared my original draft to their
;version of JAM's FF Events and Tubes and it corrected several faulty assumptions.)
;JAM for the original FF Events and Tubes. 
;Smiley for the hex tweak to skip inputs for tube breaking
;as well as other troubleshooting. 
;(Noxus for recommending skipping inputs for tube breaking.)

;Various PLMs were studied, including Tundain's Remote Gates, Oi27's Hyper Beam,
;Benox50's Extended Event PLM, as well as Amoeba and TestRunner's version 
;of JAM's FF Events and Maridia Tubes.

;Thanks to Amoeba for his feedback which encouraged me to modify the 
;Maridia tube more. The Subversion repo was crucial to understanding
;the Maridia Tube PLM better. 
 
;-----------------------------------------------------------------------------------------

lorom

;----------------------------Defines-----------------------------------------------------

!84Freespace = $84F750
!8FRoomShake = $8FF2E0	

;---------------------------Event PLM----------------------------------------------------

org !84Freespace

DW Setup
DW InstList 

Setup:

LDX $1DC7,y
LDA.w #EnemiesKilled
STA $1CD7,y
RTS

EnemiesKilled:

LDA $0E50		;Enemies killed
CMP $0E52		;Enemies needed
BCC End
LDY $1C27		;PLM index
LDA $1DC7,y		;PLM variable (event # to set)
AND #$00FF
JSL $8081FA		;Set event A
LDA $1DC7,y : AND #$F000 : CMP #$4000 : BNE .Event : BEQ .UnsetEvent

.Event

RTS

.UnsetEvent

LDY $1C27		;PLM index
LDA $1DC7,y		;PLM variable (event # to set)
AND #$00FF
JSL $808212

End: 

RTS 

InstList:

DW $86B4

;-----------------------------Maridia Tube PLM--------------------------------------------

;Allows you to use the Maridia Tube as an Event PLM.
;Use the low byte on the Maridia Tube's PLM Value in the editor to set the event.

MaridiaTube1:

PHY
LDY $1C27		;PLM index (PLM ID)
LDA $1DC7,y		;PLM variable (event # to set)
PLY
JSL $8081FA		;Set event A

MaridiaTube2:

RTS

MaridiaTube3:

PHY
LDY $1C27		;PLM index
LDA $1DC7,y		;PLM variable (event # to set)
PLY
JSL $8081FA		;Set event A
RTS

;-----------------------------Vanilla Hijacks--------------------------------------------

org $84D70E		;Maridia Tube instruction list location change
				;Removes Event Check (which was keeping us from using Event 0B.)

DW $D4DA		;Vanilla was $D4D4, this now skips the event check code.

org $84D50F 	;Maridia Tube Hijack, quite likely as JAM would have intended. 

DW MaridiaTube1,MaridiaTube2

;--------------Skip Input for Tube Break---------------------------------

;Comment this out if you want Vanilla behavior
;In Super Metroid, up, down, left, right, a,b,x, or y 
;trigger the glass breaking.
;This bypasses that behavior.

org $84D4C4 ;Hex Edit by Smiley, recommended by Noxus

NOP
NOP

;---------------Skip Disable Samus Controls-----------------------------

;Comment this out if you want Vanilla behavior.

;From Amoeba and TestRunner's version.
;Allows Samus to move freely during Tube Explosion.
;Sets Event prior to Tube Explosion

org $84D4F4

DW MaridiaTube3

;--------------------------Optional Room ASM----------------------------------------------

org !8FRoomShake

EarthQuake:

LDA $0E50  
CMP $0E52  
BCC .End

STZ $07DF

LDA #$0006			;Credits to MetroidNerd#9001 for earthquake code. 
STA $183E
LDA #$0046 
STA $1840
LDA #$0046
JSL $8090CB

.End
RTS

